//This example code is in the Public Domain (or CC0 licensed, at your option.)
//By Evandro Copercini - 2018
//
//This example creates a bridge between Serial and Classical Bluetooth (SPP)
//and also demonstrate that SerialBT have the same functionalities of a normal Serial

#include "BluetoothSerial.h"
#include <esp_task_wdt.h>
#include "SPIFFS.h"


#if !defined(CONFIG_BT_ENABLED) || !defined(CONFIG_BLUEDROID_ENABLED)
#error Bluetooth is not enabled! Please run `make menuconfig`
 to and enable it
#endif

#if !defined(CONFIG_BT_SPP_ENABLED)
#error Serial Bluetooth not available or not enabled. It is only available for the ESP32 chip.
#endif

BluetoothSerial SerialBT;

#define TENSAO1 33
#define TENSAO2 32
#define CALIBRA 25

#define WDT_TIMEOUT 3

String device_name = "MAG_BT";
int valorAD = 0;
uint32_t sendData = 0;
char comando;
bool pisca = 0;
bool aux_compara = false;
bool alerta = 0;
int tensao1 = 0, tensao2 = 0, tensao3 = 0;
int tempodelay = 500;
int offset = 0;
uint32_t compareMillis = 0;
uint32_t period_us = 0;

String cmdReceived = "";
char receivedChars[100];
int charIndex = 0;

int c1 = 0;
int c2 = 0;

TaskHandle_t task_conexao, task_envio;

void setup() {
  Serial.begin(115200);
  SerialBT.begin(device_name);  //Bluetooth device name
  Serial.printf("O dispositivo \"%s\" está iniciado.\nPronto para ser pareado...\n", device_name.c_str());
  pinMode(19, OUTPUT);  //definindo pino de saída pro LED
  pinMode(CALIBRA, INPUT);
  pinMode(TENSAO1, INPUT);  //definindo pino de entrada pro AD 1
  pinMode(TENSAO2, INPUT);  //definindo pino de entrada pro AD 2
  pinMode(21, OUTPUT);

  //testa GPIO
  digitalWrite(19, HIGH);
  delay(500);
  digitalWrite(19, LOW);

  // Inicializa o SPIFFS
  if (!SPIFFS.begin(true)) {
    Serial.println("Falha ao montar o sistema de arquivos SPIFFS");
    return;
  }

  File file = SPIFFS.open("/sensibilidade.txt", FILE_READ);
  if (!file) {
    Serial.println("Erro ao abrir o arquivo para leitura");
  }
  offset = atoi(file.readString().c_str());
  file.close();


  File calib1 = SPIFFS.open("/calib1.txt", FILE_READ);
  if (!calib1) {
    Serial.println("Erro ao abrir o arquivo para leitura");
  }
  c1 = atoi(calib1.readString().c_str());
  calib1.close();


  File calib2 = SPIFFS.open("/calib2.txt", FILE_READ);
  if (!calib2) {
    Serial.println("Erro ao abrir o arquivo para leitura");
  }

  c2 = atoi(calib2.readString().c_str());
  calib2.close();

  Serial.printf("Sensibilidade: %i - C1: %i = C2: %i\n", offset, c1, c2);

  sendData = 100;  //tempo de envio setado para 100ms
  compareMillis = millis();


  esp_task_wdt_init(WDT_TIMEOUT, true);  // enable panic so ESP32 restarts
  //esp_task_wdt_add(NULL);                // add current thread to WDT watch

  xTaskCreate(
    conexao,
    "Task Conexao",
    2000,
    NULL,
    1,
    &task_conexao  // Task handle
  );

  xTaskCreate(
    envio,
    "Task Envio",
    10000,
    NULL,
    1,
    &task_envio  // Task handle
  );

  
}

void calibrar() {

  c1 = 0;
  c2 = 0;


  for (int i = 0; i < 20; i++) {
    converte();
    c1 += tensao1;
    c2 += tensao2;
    delay(10);
  }
  c1 = (c1 / 20);
  c2 = (c2 / 20);

  Serial.printf("calibradas: c1 = %i e c2 = %i\n", c1, c2);

  char str_c1[12], str_c2[12];

  itoa(c1, str_c1, 10);
  itoa(c2, str_c2, 10);

  File calib1 = SPIFFS.open("/calib1.txt", FILE_WRITE);
  if (!calib1) {
    Serial.println("Erro ao abrir o arquivo para escrita");
    return;
  }

  if (calib1.print(str_c1)) {
    Serial.println("Valor salvo com sucesso");
  } else {
    Serial.println("Erro ao salvar o valor");
  }

  calib1.close();

  File calib2 = SPIFFS.open("/calib2.txt", FILE_WRITE);
  if (!calib2) {
    Serial.println("Erro ao abrir o arquivo para escrita");
    return;
  }

  if (calib2.print(str_c2)) {
    Serial.println("Valor salvo com sucesso");
  } else {
    Serial.println("Erro ao salvar o valor");
  }

  calib2.close();
}

void converte() {
  tensao1 = 0, tensao2 = 0;

  for (int i = 0; i < 10; i++) {
    tensao1 += 0.8247 * analogRead(TENSAO1);
    tensao2 += 0.8247 * analogRead(TENSAO2);
    delay(10);
  }
  tensao1 /= 10;
  tensao2 /= 10;

  //Serial.printf("tensão1 = %imV e tensao2 = %imV\n", tensao1, tensao2);
}

void compare() {

  if ((millis() - compareMillis) > 2000) {
    compareMillis = millis();

    if (abs((tensao1 - c1) > offset) || abs((c1 - tensao1) > offset)) {
      Serial.println("Alerta");
      alerta = true;
      //digitalWrite(21, HIGH);
    } else {
      alerta = false;
      //digitalWrite(21, LOW);
    }

    if (abs((tensao2 - c2) > offset) || abs((c2 - tensao2) > offset)) {
      alerta = true;
      Serial.println("Alerta");
      //digitalWrite(21, HIGH);
    } else {
      alerta = false;
      //digitalWrite(21, LOW);
    }
  }
}

void sendAD() {

  String v1 = "t1:" + String(tensao1) + "\n";
  String v2 = "t2:" + String(tensao2) + "\n";

  if ((millis() - sendData) > 100) {
    sendData = millis();
    converte();
    SerialBT.print(v1);
    SerialBT.print(v2);
  }
}

void receive() {

  while (SerialBT.available() > 0) {
    char receivedChar = SerialBT.read();

    if (receivedChar == '\n') {
      receivedChars[charIndex] = '\0';

      String receivedString = String(receivedChars);

      Serial.println("String received: " + receivedString);
      charIndex = 0;

      if (receivedString.indexOf("on") > -1) {
        digitalWrite(19, HIGH);
        Serial.println("HIGH");
        pisca = false;
      }

      if (receivedString.indexOf("off") > -1) {
        digitalWrite(19, LOW);
        Serial.println("LOW");
        pisca = false;
      }
      if (receivedString.indexOf("pisca") > -1) {
        pisca = true;
        Serial.println("pisca");
      }

      if (receivedString.indexOf("startread") > -1) {
        aux_compara = true;
        Serial.println("compara = true");
      }

      if (receivedString.indexOf("stopread") > -1) {
        aux_compara = false;
        Serial.println("compara = false");
      }

      if (receivedString.indexOf("sensibilidade:") > -1) {
        int index = receivedString.indexOf(":");

        cmdReceived = receivedString.substring(index + 1);

        Serial.print("cmd: ");
        Serial.println(cmdReceived);

        File file = SPIFFS.open("/sensibilidade.txt", FILE_WRITE);
        if (!file) {


          Serial.println("Erro ao abrir o arquivo para escrita");
          return;
        }

        if (file.print(cmdReceived)) {
          Serial.println("Valor salvo com sucesso");
        } else {
          Serial.println("Erro ao salvar o valor");
        }

        file.close();

        offset = cmdReceived.toInt();
        Serial.printf("offset: %i\n", offset);
      }

      if (receivedString.indexOf("freq:") > -1) {
        int index = receivedString.indexOf(":");

        cmdReceived = receivedString.substring(index + 1);

        Serial.print("cmd: ");
        Serial.println(cmdReceived);

        tempodelay = (500 / cmdReceived.toInt());
      }

    } else {
      // Adiciona o caractere recebido ao buffer
      receivedChars[charIndex] = receivedChar;
      charIndex++;

      // Verifica se o buffer está cheio para evitar overflow
      if (charIndex >= sizeof(receivedChars) - 1) {
        charIndex = sizeof(receivedChars) - 1;
      }
    }
  }
}

void envio(void* parameter) {
  while (1) {
    delay(0);
    yield();
    esp_task_wdt_reset();

    if (!digitalRead(CALIBRA)) {
      Serial.println("calibração...");
      delay(200);
      calibrar();
    }

    sendAD();

    receive();


    if (aux_compara) {
      compare();
    } else {
      alerta = false;
    }

    if (pisca) {
      digitalWrite(19, HIGH);
      delay(tempodelay);
      digitalWrite(19, LOW);
      delay(tempodelay);
    }
    
  }
}

void conexao(void* parameter) {
  while (1) {
    delay(0);
    yield();
    esp_task_wdt_reset();

    if (alerta) {
      
      period_us = ((analogRead(TENSAO1) * (33.0 / 1000.0)) * 3.33);

      digitalWrite(21, HIGH);
      delayMicroseconds(period_us);
      digitalWrite(21, LOW);
      delayMicroseconds((333.0 - period_us));
      
      period_us = (analogRead(TENSAO1) * 1.17);

      digitalWrite(21, HIGH);
      delayMicroseconds(period_us);
      digitalWrite(21, LOW);
      delayMicroseconds((period_us));
    }
    
  }
}

void loop() {
  vTaskDelete(NULL);
}
